/**
 * Created by anthony on 12/09/2016.
 */
var gulp = require('gulp');
var tap = require('gulp-tap');
var app;
var path = require('path');
var _ = require('lodash');
var rename = require('gulp-rename');
var Stream = require('stream');
var merge = require('merge-stream');
var appName;
var appBuildPath;

function template(options) {

    var stream = new Stream.Transform({objectMode: true});

    stream._transform = function (file, unused, callback) {
        var contents = String(file.contents);
        var compiled = _.template(contents);
        var result = compiled(options);

        file.contents = new Buffer(result);
        callback(null, file);
    };

    return stream;
}

gulp.task('android:copy-project', function () {

    app = require('../../lib').app;
    appName = _.snakeCase(app.name);
    appBuildPath = './build/' + appName;

    return gulp.src('./templates/project/**/*', {cwd: 'platforms/android'})
        .pipe(gulp.dest(appBuildPath));
});

gulp.task('android:create-activities', ['android:copy-project'], function () {
    var destPath = path.join('app/src/main/java', _.chain(app.package_id).split('.').join('/').value());
    var opts = {
        package_id: app.package_id,
        import_activity_base: 'de.fluidco.jnandroidruntime.JnActivityBase',
        activity_base_class: 'JnActivityBase'
    };

    var streams = [];
    var allPages = _.keys(app.screens.default);
    _.each(allPages, function (pageName) {
        var className = _.camelCase(pageName);
        var pipe = gulp.src('./templates/activity.txt', {cwd: 'platforms/android'})
            .pipe(rename(className + '.java'))
            .pipe(template(_.extend({activity_name: className, page_name:pageName}, opts)))
            .pipe(gulp.dest(path.join(appBuildPath, destPath)));

        streams.push(pipe);
    });

    return merge(streams);

});

gulp.task('android:expand-project-templates', ['android:android:create-activities'], function () {
    var env = {
        package_id: app.package_id,
        runtime_library: 'de.fluidco.jnandroidruntime:jn-android-runtime:0.1.1',
        app_name: app.name
    };
});

gulp.task('platform:android', ['android:create-activities'], function () {
    console.log('app name:', appName);

});